@@ -2,7 +2,7 @@
  *	avarice - The "avarice" program.
  *	Copyright (C) 2001 Scott Finneran
  *	Copyright (C) 2002, 2003, 2004 Intel Corporation
- *	Copyright (C) 2005 Joerg Wunsch
+ *	Copyright (C) 2005,2006 Joerg Wunsch
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License Version 2
@@ -19,7 +19,7 @@
  *
  * This file contains functions for interfacing with the GDB remote protocol.
  *
- * $Id: jtag.h,v 1.21 2005/08/15 18:47:03 joerg_wunsch Exp $
+ * $Id: jtag.h,v 1.24 2006/08/08 21:27:34 joerg_wunsch Exp $
  */
 
 #ifndef JTAG_H
@@ -226,6 +226,9 @@ enum
     FUSE_CKSEL1                       = 0x02,
     FUSE_CKSEL0                       = 0x01,
 
+    MAX_FLASH_PAGE_SIZE               = 256,
+    MAX_EEPROM_PAGE_SIZE              = 16,
+
     // JTAG ICE mkI protocol constants
 
     // Address space selector values
@@ -602,6 +605,14 @@ class jtag
   // Name of the device controlled by the JTAG ICE
   char *device_name;
 
+  // Daisy chain info
+  struct {
+    unsigned char units_before;
+    unsigned char units_after;
+    unsigned char bits_before;
+    unsigned char bits_after;
+  } dchain;
+
   protected:
   pid_t openUSB(const char *jtagDeviceName);
   int safewrite(const void *b, int count);
@@ -626,7 +637,7 @@ class jtag
 
   public:
   jtag(void);
-  jtag(const char *dev);
+  jtag(const char *dev, char *name);
   virtual ~jtag(void);
 
   // Basic JTAG I/O
