@@ -2,7 +2,7 @@
  *	avarice - The "avarice" program.
  *	Copyright (C) 2001 Scott Finneran
  *      Copyright (C) 2002, 2003, 2004 Intel Corporation
- *	Copyright (C) 2005 Joerg Wunsch
+ *	Copyright (C) 2005,2006 Joerg Wunsch
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License Version 2
@@ -19,7 +19,7 @@
  *
  * This file implements the basic IO handling for the mkII protocol.
  *
- * $Id: jtag2io.cc,v 1.4 2005/11/17 18:34:25 idgay Exp $
+ * $Id: jtag2io.cc,v 1.7 2006/10/08 19:13:08 joerg_wunsch Exp $
  */
 
 
@@ -464,7 +464,7 @@ void jtag2::startJtagLink(void)
 
     for (unsigned int i = 0; i < sizeof bitrates / sizeof *bitrates; i++)
 	if (synchroniseAt(bitrates[i])) {
-	    uchar val = EMULATOR_MODE_JTAG;
+	    uchar val = useDebugWire? EMULATOR_MODE_DEBUGWIRE: EMULATOR_MODE_JTAG;
 	    setJtagParameter(PAR_EMULATOR_MODE, &val, 1);
 	    signedIn = true;
 
@@ -492,20 +492,35 @@ void jtag2::deviceAutoConfig(void)
     // Auto config
     debugOut("Automatic device detection: ");
 
+    /* Set daisy chain information */
+    configDaisyChain();
+
     /* Read in the JTAG device ID to determine device */
-    getJtagParameter(PAR_JTAGID, resp, respSize);
-    jtagCheck(respSize == 4);
-    device_id = resp[1] | (resp[2] << 8) | (resp[3] << 16) | resp[4] << 24;
-    delete [] resp;
+    if (useDebugWire)
+    {
+	getJtagParameter(PAR_TARGET_SIGNATURE, resp, respSize);
+	jtagCheck(respSize == 2);
+	device_id = resp[1] | (resp[2] << 8);
+	delete [] resp;
 
-    debugOut("JTAG id = 0x%0X : Ver = 0x%0x : Device = 0x%0x : Manuf = 0x%0x\n",
-             device_id,
-             (device_id & 0xF0000000) >> 28,
-             (device_id & 0x0FFFF000) >> 12,
-             (device_id & 0x00000FFE) >> 1);
+	statusOut("Reported debugWire device ID: 0x%0X\n", device_id);
+    }
+    else
+    {
+	getJtagParameter(PAR_JTAGID, resp, respSize);
+	jtagCheck(respSize == 4);
+	device_id = resp[1] | (resp[2] << 8) | (resp[3] << 16) | resp[4] << 24;
+	delete [] resp;
 
-    device_id = (device_id & 0x0FFFF000) >> 12;
-    statusOut("Reported JTAG device ID: 0x%0X\n", device_id);
+	debugOut("JTAG id = 0x%0X : Ver = 0x%0x : Device = 0x%0x : Manuf = 0x%0x\n",
+		 device_id,
+		 (device_id & 0xF0000000) >> 28,
+		 (device_id & 0x0FFFF000) >> 12,
+		 (device_id & 0x00000FFE) >> 1);
+
+	device_id = (device_id & 0x0FFFF000) >> 12;
+	statusOut("Reported JTAG device ID: 0x%0X\n", device_id);
+    }
 
     if (device_name == 0)
     {
@@ -585,64 +600,86 @@ void jtag2::initJtagOnChipDebugging(unsigned long bitrate)
 {
     statusOut("Preparing the target device for On Chip Debugging.\n");
 
-    uchar br;
-    if (bitrate >= 6400000)
-      br = 0;
-    else if (bitrate >= 2800000)
-      br = 1;
-    else if (bitrate >= 20900)
-      br = (unsigned char)(5.35e6 / (double)bitrate);
-    else
-      br = 255;
-    // Set JTAG bitrate
-    setJtagParameter(PAR_OCD_JTAG_CLK, &br, 1);
-
-    // When attaching we can't change fuse bits, etc, as
-    // enabling+disabling programming resets the processor
-    enableProgramming();
-
-    // Ensure that all lock bits are "unlocked" ie all 1's
-    uchar *lockBits = 0;
-    lockBits = jtagRead(LOCK_SPACE_ADDR_OFFSET + 0, 1);
-
-    if (*lockBits != LOCK_BITS_ALL_UNLOCKED)
+    // debugWire cannot read or manipulate fuse or lock bits
+    if (!useDebugWire)
     {
-        lockBits[0] = LOCK_BITS_ALL_UNLOCKED;
-        jtagWrite(LOCK_SPACE_ADDR_OFFSET + 0, 1, lockBits);
-    }
+      uchar br;
+      if (bitrate >= 6400000)
+	br = 0;
+      else if (bitrate >= 2800000)
+	br = 1;
+      else if (bitrate >= 20900)
+	br = (unsigned char)(5.35e6 / (double)bitrate);
+      else
+	br = 255;
+      // Set JTAG bitrate
+      setJtagParameter(PAR_OCD_JTAG_CLK, &br, 1);
 
-    statusOut("\nDisabling lock bits:\n");
-    statusOut("  LockBits -> 0x%02x\n", *lockBits);
 
-    if (lockBits)
-    {
-        delete [] lockBits;
-        lockBits = 0;
-    }
+      // When attaching we can't change fuse bits, etc, as
+      // enabling+disabling programming resets the processor
+      enableProgramming();
 
-    // Ensure on-chip debug enable fuse is enabled ie '0'
-    uchar *fuseBits = 0;
-    statusOut("\nEnabling on-chip debugging:\n");
-    fuseBits = jtagRead(FUSE_SPACE_ADDR_OFFSET + 0, 3);
+      // Ensure that all lock bits are "unlocked" ie all 1's
+      uchar *lockBits = 0;
+      lockBits = jtagRead(LOCK_SPACE_ADDR_OFFSET + 0, 1);
 
-    if ((fuseBits[1] & FUSE_OCDEN) == FUSE_OCDEN)
-    {
-        fuseBits[1] = fuseBits[1] & ~FUSE_OCDEN; // clear bit
+      if (*lockBits != LOCK_BITS_ALL_UNLOCKED)
+      {
+	  lockBits[0] = LOCK_BITS_ALL_UNLOCKED;
+	  jtagWrite(LOCK_SPACE_ADDR_OFFSET + 0, 1, lockBits);
+      }
+
+      statusOut("\nDisabling lock bits:\n");
+      statusOut("  LockBits -> 0x%02x\n", *lockBits);
+
+      if (lockBits)
+      {
+	  delete [] lockBits;
+	  lockBits = 0;
+      }
+
+      // Ensure on-chip debug enable fuse is enabled ie '0'
+      uchar *fuseBits = 0;
+      statusOut("\nEnabling on-chip debugging:\n");
+      fuseBits = jtagRead(FUSE_SPACE_ADDR_OFFSET + 0, 3);
+
+      if ((fuseBits[1] & FUSE_OCDEN) == FUSE_OCDEN)
+      {
+	  fuseBits[1] = fuseBits[1] & ~FUSE_OCDEN; // clear bit
         jtagWrite(FUSE_SPACE_ADDR_OFFSET + 1, 1, &fuseBits[1]);
+      }
+
+      jtagDisplayFuses(fuseBits);
+
+      if (fuseBits)
+      {
+	  delete [] fuseBits;
+	  fuseBits = 0;
+      }
+
+      disableProgramming();
     }
 
-    jtagDisplayFuses(fuseBits);
-
-    if (fuseBits)
-    {
-        delete [] fuseBits;
-        fuseBits = 0;
-    }
-
-    disableProgramming();
 
     resetProgram();
     uchar timers = 0;		// stopped
     setJtagParameter(PAR_TIMERS_RUNNING, &timers, 1);
     resetProgram();
 }
+
+void jtag2::configDaisyChain(void)
+{
+    unsigned char buf[4];
+
+    if ((dchain.units_before > 0) ||
+	(dchain.units_after > 0) ||
+	(dchain.bits_before > 0) ||
+	(dchain.bits_after > 0) ){
+	buf[0] = dchain.units_before;
+	buf[1] = dchain.units_after;
+	buf[2] = dchain.bits_before;
+	buf[3] = dchain.bits_after;
+	setJtagParameter(PAR_DAISY_CHAIN_INFO, buf, 4);
+    }
+}
