@@ -1,8 +1,9 @@
+
 /*
  *	avarice - The "avarice" program.
  *	Copyright (C) 2001 Scott Finneran
  *	Copyright (C) 2002, 2003, 2004 Intel Corporation
- *	Copyright (C) 2005 Joerg Wunsch
+ *	Copyright (C) 2005,2006 Joerg Wunsch
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License Version 2
@@ -20,7 +21,7 @@
  *
  * This file contains the main() & support for avrjtagd.
  *
- * $Id: main.cc,v 1.31 2005/05/27 20:42:13 joerg_wunsch Exp $
+ * $Id: main.cc,v 1.37 2006/10/08 19:16:18 joerg_wunsch Exp $
  */
 
 
@@ -135,27 +136,6 @@ static void usage(const char *progname)
 	    "  -1, --mkI                   Connect to JTAG ICE mkI (default)\n");
     fprintf(stderr,
 	    "  -2, --mkII                  Connect to JTAG ICE mkII\n");
-    fprintf(stderr,
-	    "  -d, --debug                 Enable printing of debug"
-	    " information.\n");
-    fprintf(stderr,
-	    "  -D, --detach                Detach once synced with JTAG ICE\n");
-    fprintf(stderr,
-	    "  -C, --capture               Capture running program.\n"
-	    "                                Note: debugging must have been enabled prior\n"
-            "                                to starting the program. (e.g., by running\n"
-            "                                avarice earlier)\n");
-    fprintf(stderr,
-	    "  -I, --ignore-intr           Automatically step over interrupts.\n"
-	    "                                Note: EXPERIMENTAL. Can not currently handle\n"
-            "                                devices fused for compatibility.\n");
-    fprintf(stderr,
-	    "  -f, --file <filename>       Specify a file for use with the --program and\n"
-            "                                --verify options. If --file is passed and\n"
-            "                                neither --program or --verify are given then\n"
-            "                                --program is implied.\n");
-    fprintf(stderr,
-	    "  -j, --jtag <devname>        Port attached to JTAG box (default: /dev/avrjtag).\n");
     fprintf(stderr,
             "  -B, --jtag-bitrate <rate>   Set the bitrate that the JTAG box communicates\n"
             "                                with the avr target device. This must be less\n"
@@ -163,30 +143,59 @@ static void usage(const char *progname)
             "                                values are 1MHz, 500KHz, 250KHz or 125KHz.\n"
             "                                (default: 1MHz)\n");
     fprintf(stderr,
-	    "  -p, --program               Program the target.\n"
-	    "                                Binary filename must be specified with --file\n"
-	    "                                option.\n");
+	    "  -C, --capture               Capture running program.\n"
+	    "                                Note: debugging must have been enabled prior\n"
+            "                                to starting the program. (e.g., by running\n"
+            "                                avarice earlier)\n");
     fprintf(stderr,
-            "  -v, --verify                Verify program in device against file specified\n"
-            "                                with --file option.\n");
+	    "  -c, --daisy-chain <ub,ua,bb,ba> Daisy chain settings:\n"
+	    "                                <units before, units after,\n"
+	    "                                bits before, bits after>\n");
+    fprintf(stderr,
+	    "  -D, --detach                Detach once synced with JTAG ICE\n");
+    fprintf(stderr,
+	    "  -d, --debug                 Enable printing of debug information.\n");
     fprintf(stderr,
             "  -e, --erase                 Erase target.\n");
     fprintf(stderr,
-            "  -r, --read-fuses            Read fuses bytes.\n");
+	    "  -f, --file <filename>       Specify a file for use with the --program and\n"
+            "                                --verify options. If --file is passed and\n"
+            "                                neither --program or --verify are given then\n"
+            "                                --program is implied.\n");
     fprintf(stderr,
-            "  -W, --write-fuses <eehhll>  Write fuses bytes.\n");
+	    "  -I, --ignore-intr           Automatically step over interrupts.\n"
+	    "                                Note: EXPERIMENTAL. Can not currently handle\n"
+            "                                devices fused for compatibility.\n");
     fprintf(stderr,
-            "  -l, --read-lockbits         Read lock bits.\n");
+	    "  -j, --jtag <devname>        Port attached to JTAG box (default: /dev/avrjtag).\n");
     fprintf(stderr,
             "  -L, --write-lockbits <ll>   Write lock bits.\n");
+    fprintf(stderr,
+            "  -l, --read-lockbits         Read lock bits.\n");
     fprintf(stderr,
             "  -P, --part <name>           Target device name (e.g."
             " atmega16)\n\n");
+    fprintf(stderr,
+	    "  -p, --program               Program the target.\n"
+	    "                                Binary filename must be specified with --file\n"
+	    "                                option.\n");
+    fprintf(stderr,
+            "  -r, --read-fuses            Read fuses bytes.\n");
+    fprintf(stderr,
+	    "  -V, --version               Print version information.\n");
+    fprintf(stderr,
+            "  -v, --verify                Verify program in device against file specified\n"
+            "                                with --file option.\n");
+    fprintf(stderr,
+	    "  -w, --debugwire             For the JTAG ICE mkII, connect to the target\n"
+	    "                                using debugWire protocol rather than JTAG.\n");
+    fprintf(stderr,
+            "  -W, --write-fuses <eehhll>  Write fuses bytes.\n");
     fprintf(stderr,
 	    "HOST_NAME defaults to 0.0.0.0 (listen on any interface).\n"
 	    "\":PORT\" is required to put avarice into gdb server mode.\n\n");
     fprintf(stderr,
-	    "e.g. %s --program --file test.bin --jtag /dev/ttyS0 :4242\n\n",
+	    "e.g. %s --erase --program --file test.bin --jtag /dev/ttyS0 :4242\n\n",
 	    progname);
     exit(1);
 }
@@ -195,23 +204,25 @@ static struct option long_opts[] = {
     /* name,                 has_arg, flag,   val */
     { "mkI",                 0,       0,     '1' },
     { "mkII",                0,       0,     '2' },
-    { "help",                0,       0,     'h' },
-    { "debug",               0,       0,     'd' },
-    { "version",             0,       0,     'V' },
-    { "program",             0,       0,     'p' },
-    { "verify",              0,       0,     'v' },
-    { "erase",               0,       0,     'e' },
-    { "read-fuses",          0,       0,     'r' },
-    { "write-fuses",         1,       0,     'W' },
-    { "read-lockbits",       0,       0,     'l' },
-    { "write-lockbits",      1,       0,     'L' },
-    { "part",                1,       0,     'P' },
-    { "jtag",                1,       0,     'j' },
     { "jtag-bitrate",        1,       0,     'B' },
-    { "ignore-intr",         0,       0,     'I' },
-    { "detach",              0,       0,     'D' },
     { "capture",             0,       0,     'C' },
+    { "daisy-chain",         1,       0,     'c' },
+    { "detach",              0,       0,     'D' },
+    { "debug",               0,       0,     'd' },
+    { "erase",               0,       0,     'e' },
     { "file",                1,       0,     'f' },
+    { "help",                0,       0,     'h' },
+    { "ignore-intr",         0,       0,     'I' },
+    { "jtag",                1,       0,     'j' },
+    { "write-lockbits",      1,       0,     'L' },
+    { "read-lockbits",       0,       0,     'l' },
+    { "part",                1,       0,     'P' },
+    { "program",             0,       0,     'p' },
+    { "read-fuses",          0,       0,     'r' },
+    { "version",             0,       0,     'V' },
+    { "verify",              0,       0,     'v' },
+    { "debugwire",           0,       0,     'w' },
+    { "write-fuses",         1,       0,     'W' },
     { 0,                     0,       0,      0 }
 };
 
@@ -241,8 +252,14 @@ int main(int argc, char **argv)
     bool capture = false;
     bool verify = false;
     char *progname = argv[0];
-    int protocol = 1;		// default to mkI protocol
+    enum {
+	MKI, MKII, MKII_DW
+    } protocol = MKI;		// default to mkI protocol
     int  option_index;
+    unsigned int units_before = 0;
+    unsigned int units_after = 0;
+    unsigned int bits_before = 0;
+    unsigned int bits_after = 0;
 
     statusOut("AVaRICE version %s, %s %s\n\n",
 	      PACKAGE_VERSION, __DATE__, __TIME__);
@@ -253,7 +270,7 @@ int main(int argc, char **argv)
 
     while (1)
     {
-        int c = getopt_long (argc, argv, "12VhdDCIf:j:B:pverW:lL:P:",
+        int c = getopt_long (argc, argv, "12B:Cc:Ddef:hIj:L:lP:prVvwW:",
                              long_opts, &option_index);
         if (c == -1)
             break;              /* no more options */
@@ -263,60 +280,78 @@ int main(int argc, char **argv)
             case '?':
                 usage(progname);
 	    case '1':
-		protocol = 1;
+		protocol = MKI;
 		break;
 	    case '2':
-		protocol = 2;
+		// If we've already seen a -w option, don't revert to -2.
+		if (protocol != MKII_DW)
+		    protocol = MKII;
 		break;
-            case 'd':
-                debugMode = true;
-                break;
-            case 'D':
-                detach = true;
+            case 'B':
+		jtagBitrate = parseJtagBitrate(optarg);
                 break;
             case 'C':
                 capture = true;
                 break;
-            case 'I':
-                ignoreInterrupts = true;
+            case 'c':
+                if (sscanf(optarg,"%u,%u,%u,%u",
+                           &units_before, &units_after,
+                           &bits_before, &bits_after) != 4)
+                    usage(progname);
+		if (units_before > 255 || units_after > 255 ||
+		    bits_before > 7 || bits_after > 7) {
+		    fprintf(stderr,
+			    "%s: daisy-chain parameters out of range\n",
+			    progname);
+		    exit(1);
+		}
                 break;
-            case 'f':
-                inFileName = optarg;
+            case 'D':
+                detach = true;
                 break;
-            case 'j':
-                jtagDeviceName = optarg;
-                break;
-            case 'B':
-		jtagBitrate = parseJtagBitrate(optarg);
-                break;
-            case 'p':
-                program = true;
-                break;
-            case 'v':
-                verify = true;
+            case 'd':
+                debugMode = true;
                 break;
             case 'e':
                 erase = true;
                 break;
-            case 'r':
-                readFuses = true;
+            case 'f':
+                inFileName = optarg;
                 break;
-            case 'W':
-                fuses = optarg;
-                writeFuses = true;
+            case 'I':
+                ignoreInterrupts = true;
                 break;
-            case 'l':
-                readLockBits = true;
+            case 'j':
+                jtagDeviceName = optarg;
                 break;
             case 'L':
                 lockBits = optarg;
                 writeLockBits = true;
                 break;
+            case 'l':
+                readLockBits = true;
+                break;
             case 'P':
                 device_name = optarg;
                 break;
-	    case 'V':
-		exit(0);
+            case 'p':
+                program = true;
+                break;
+            case 'r':
+                readFuses = true;
+                break;
+            case 'V':
+                exit(0);
+            case 'v':
+                verify = true;
+                break;
+            case 'w':
+	        protocol = MKII_DW;
+		break;
+            case 'W':
+                fuses = optarg;
+                writeFuses = true;
+                break;
             default:
                 fprintf (stderr, "getop() did something screwey");
                 exit (1);
@@ -392,21 +427,25 @@ int main(int argc, char **argv)
     try {
 	// And say hello to the JTAG box
 	switch (protocol) {
-	case 1:
-	    theJtagICE = new jtag1(jtagDeviceName);
+	case MKI:
+	    theJtagICE = new jtag1(jtagDeviceName, device_name);
 	    break;
 
-	case 2:
-	    theJtagICE = new jtag2(jtagDeviceName);
+	case MKII:
+	    theJtagICE = new jtag2(jtagDeviceName, device_name);
 	    break;
 
-	default:
-	    fprintf(stderr,
-		    "Unknonw JTAG ICE protocol: %d\n",
-		    protocol);
-	    exit(1);
+	case MKII_DW:
+	    theJtagICE = new jtag2(jtagDeviceName, device_name, true);
+	    break;
 	}
 
+	// Set Daisy-chain variables
+	theJtagICE->dchain.units_before = (unsigned char) units_before;
+	theJtagICE->dchain.units_after = (unsigned char) units_after;
+	theJtagICE->dchain.bits_before = (unsigned char) bits_before;
+	theJtagICE->dchain.bits_after = (unsigned char) bits_after;
+
 	// Init JTAG box.
 	theJtagICE->initJtagBox();
     }
